@@ -70,26 +70,71 @@ _gcr_debug_flag_is_set (GcrDebugFlags flag)
 	return (flag & current_flags) != 0;
 }
 
+static void
+on_gcr_log_debug (const gchar *log_domain,
+                  GLogLevelFlags log_level,
+                  const gchar *message,
+                  gpointer user_data)
+{
+	GString *gstring;
+	const gchar *progname;
+
+	gstring = g_string_new (NULL);
+
+	progname = g_get_prgname ();
+	g_string_append_printf (gstring, "(%s:%lu): %s-DEBUG: %s\n",
+	                        progname ? progname : "process",
+	                        (gulong)getpid (), log_domain,
+	                        message ? message : "(NULL) message");
+
+	write (1, gstring->str, gstring->len);
+	g_string_free (gstring, TRUE);
+}
+
 void
 _gcr_debug_message (GcrDebugFlags flag, const gchar *format, ...)
 {
 	static gsize initialized_flags = 0;
-	gchar *message;
+	const gchar *messages_env;
+	const gchar *debug_env;
 	va_list args;
 
 	if (g_once_init_enter (&initialized_flags)) {
-		_gcr_debug_set_flags (g_getenv ("GCR_DEBUG"));
+		messages_env = g_getenv ("G_MESSAGES_DEBUG");
+		debug_env = g_getenv ("GCR_DEBUG");
+#ifdef GCR_DEBUG
+		if (debug_env == NULL)
+			debug_env = G_STRINGIFY (GCR_DEBUG);
+#endif
+
+		/*
+		 * If the caller is selectively asking for certain debug
+		 * messages with the GCR_DEBUG environment variable, then
+		 * we install our own output handler and only print those
+		 * messages. This happens irrespective of G_MESSAGES_DEBUG
+		 */
+		if (messages_env == NULL && debug_env != NULL)
+			g_log_set_handler (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+			                   on_gcr_log_debug, NULL);
+
+		/*
+		 * If the caller is using G_MESSAGES_DEBUG then we enable
+		 * all our debug messages, and let Glib filter which ones
+		 * to display.
+		 */
+		if (messages_env != NULL && debug_env == NULL)
+			debug_env = "all";
+
+		_gcr_debug_set_flags (debug_env);
+
 		g_once_init_leave (&initialized_flags, 1);
 	}
 
-	va_start (args, format);
-	message = g_strdup_vprintf (format, args);
-	va_end (args);
-
-	if (flag & current_flags)
-		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s", message);
-
-	g_free (message);
+	if (flag & current_flags) {
+		va_start (args, format);
+		g_logv (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format, args);
+		va_end (args);
+	}
 }
 
 #else /* !WITH_DEBUG */
